"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var timer_error_1 = __importDefault(require("./timer-error"));
var TimerValidator = (function () {
    function TimerValidator() {
    }
    TimerValidator.prototype.invalidDateString = function (date) {
        return isNaN(Date.parse(date));
    };
    TimerValidator.prototype.invalidWeekDay = function (day) {
        return (day < 0 || day > 6);
    };
    TimerValidator.prototype.invalidTime = function (time) {
        var TIME_REGEX = /^$|^(([01][0-9])|(2[0-3])):[0-5][0-9]$/;
        return !TIME_REGEX.test(time);
    };
    TimerValidator.prototype.invalidRange = function (from, to) {
        var fromTime = new Date().setHours(Number(from.split(":")[0]), Number(from.split(":")[1]));
        var toTime = new Date().setHours(Number(to.split(":")[0]), Number(to.split(":")[1]));
        return toTime < fromTime;
    };
    TimerValidator.prototype.validateVacations = function (vacations) {
        var _this = this;
        vacations.forEach(function (vacation) {
            if (_this.invalidDateString(vacation)) {
                throw new timer_error_1.default("Invalid Vacation Date \"" + vacation + "\" !");
            }
        });
    };
    TimerValidator.prototype.validateNormalWorkingDays = function (workingDays) {
        var _this = this;
        var weekDays = Object.keys(workingDays);
        if (!workingDays || weekDays.length === 0) {
            throw new timer_error_1.default('You should provide at least one working day !');
        }
        weekDays.forEach(function (d) {
            if (!d || isNaN(Number(d)) || _this.invalidWeekDay(Number(d))) {
                throw new timer_error_1.default("Invalid Working Day \"" + d + "\" => Day should be from 0 to 6 !");
            }
            if (workingDays[Number(d)].length == 0) {
                throw new timer_error_1.default("Invalid Working Day \"" + d + "\" => Day should contain one working minute at least !");
            }
            workingDays[Number(d)].forEach(function (w) {
                if (!w.from || !w.to || _this.invalidTime(w.from) || _this.invalidTime(w.to)) {
                    throw new timer_error_1.default("Invalid Working Hours \"From: " + w.from + " To: " + w.to + "\" => Time should be hh:mm !");
                }
                if (_this.invalidRange(w.from, w.to)) {
                    throw new timer_error_1.default("Invalid Working Hours \"From: " + w.from + " To: " + w.to + "\" => To should be greater than from !");
                }
            });
        });
    };
    TimerValidator.prototype.validateExceptionalWorkingDays = function (exceptionalWorkingHours) {
        var _this = this;
        var dates = Object.keys(exceptionalWorkingHours);
        dates.forEach(function (day) {
            if (_this.invalidDateString(day)) {
                throw new timer_error_1.default("Invalid Exceptional Date \"" + day + "\" !");
            }
            if (exceptionalWorkingHours[day].length == 0) {
                throw new timer_error_1.default("Invalid Exceptional Working Day \"" + day + "\" => Day should contain one working minute at least !");
            }
            exceptionalWorkingHours[day].forEach(function (w) {
                if (!w.from || !w.to || _this.invalidTime(w.from) || _this.invalidTime(w.to)) {
                    throw new timer_error_1.default("Invalid Exceptional Working Hours \"From: " + w.from + " To: " + w.to + "\" => Time should be hh:mm !");
                }
                if (_this.invalidRange(w.from, w.to)) {
                    throw new timer_error_1.default("Invalid Exceptional Working Hours \"From: " + w.from + " To: " + w.to + "\" => To should be greater than from !");
                }
            });
        });
    };
    TimerValidator.prototype.validateMinBuffer = function (minBufferedDays) {
        if (isNaN(minBufferedDays)) {
            throw new timer_error_1.default('Invalid Minimum Buffered Days => Should be number !');
        }
    };
    TimerValidator.prototype.validateMaxBuffer = function (maxBufferedDays) {
        if (isNaN(maxBufferedDays)) {
            throw new timer_error_1.default('Invalid Maximum Buffered Days => Should be number !');
        }
    };
    return TimerValidator;
}());
exports.TimerValidator = TimerValidator;
