"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var timer_validator_1 = require("./timer-validator");
var timer_unit_1 = require("./timer-unit");
var i_business_day_1 = require("./interfaces/i-business-day");
var timer_error_1 = __importDefault(require("./timer-error"));
var working_timeout_1 = __importDefault(require("./working-timeout"));
var timer_utils_1 = require("./timer-utils");
var Timer = (function () {
    function Timer() {
        this.bufferedCalendar = new Map();
        this.timerValidator = new timer_validator_1.TimerValidator();
    }
    Timer.prototype.setConfig = function (timerParams) {
        this.timerParams = timerParams;
        try {
            this.timerValidator.validateVacations(timerParams.vacations);
            this.timerValidator.validateNormalWorkingDays(timerParams.normalWorkingHours);
            this.timerValidator.validateExceptionalWorkingDays(timerParams.exceptionalWorkingHours);
            this.timerValidator.validateMinBuffer(timerParams.minBufferedDays);
            this.timerValidator.validateMaxBuffer(timerParams.maxBufferedDays);
            this.constructWorkingDays(timerParams, false);
            return this;
        }
        catch (error) {
            throw new timer_error_1.default(error.message);
        }
    };
    Timer.prototype.setConfigAsync = function (timerParams) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.setConfig(timerParams);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.constructWorkingDays = function (timerParams, extend) {
        var today = new Date().setHours(0, 0, 0, 0);
        var startDate = today - (timer_unit_1.TimerUnit.DAYS * Math.abs(timerParams.minBufferedDays));
        var endDate = today + (timer_unit_1.TimerUnit.DAYS * Math.abs(timerParams.maxBufferedDays));
        var processedCalendar = new Map();
        var append = null;
        if (extend) {
            if (startDate < this.minBufferedDate) {
                endDate = this.minBufferedDate - timer_unit_1.TimerUnit.DAYS;
                this.minBufferedDate = startDate;
                append = 'start';
            }
            if (endDate > this.maxBufferedDate) {
                startDate = this.maxBufferedDate + timer_unit_1.TimerUnit.DAYS;
                this.maxBufferedDate = endDate;
                append = 'end';
            }
        }
        else {
            this.minBufferedDate = startDate;
            this.maxBufferedDate = endDate;
        }
        for (var i = startDate; i <= endDate; i += timer_unit_1.TimerUnit.DAYS) {
            var date = new Date(i);
            var FULLDAY = timer_utils_1.TimerUtils.getFormatedDate(date);
            var WILDCARDDAY = '*' + FULLDAY.substr(4);
            var BUSINESSDAY = new i_business_day_1.BusinessDay(false, false, false, []);
            if (timerParams.normalWorkingHours[date.getDay()]) {
                BUSINESSDAY.workingHours = timerParams.normalWorkingHours[date.getDay()].sort(function (a, b) {
                    return a.from > b.from ? 1 : (a.from == b.from && a.to > b.to) ? 1 : -1;
                });
            }
            else {
                BUSINESSDAY.isWeekend = true;
            }
            if (timerParams.exceptionalWorkingHours[FULLDAY] || timerParams.exceptionalWorkingHours[WILDCARDDAY]) {
                BUSINESSDAY.isExceptional = true;
                BUSINESSDAY.workingHours = (timerParams.exceptionalWorkingHours[FULLDAY]) ?
                    timerParams.exceptionalWorkingHours[FULLDAY] :
                    timerParams.exceptionalWorkingHours[WILDCARDDAY];
            }
            if (timerParams.vacations.includes(FULLDAY) || timerParams.vacations.includes(WILDCARDDAY)) {
                BUSINESSDAY.isVacation = true;
                BUSINESSDAY.workingHours = [];
            }
            processedCalendar.set(FULLDAY, BUSINESSDAY);
        }
        if (append) {
            var oldBuffered = new Map(this.bufferedCalendar);
            this.bufferedCalendar.clear();
            if (append == 'start') {
                this.addToBuffer(processedCalendar);
                this.addToBuffer(oldBuffered);
            }
            if (append == 'end') {
                this.addToBuffer(oldBuffered);
                this.addToBuffer(processedCalendar);
            }
        }
        else {
            this.bufferedCalendar = processedCalendar;
        }
    };
    Timer.prototype.addToBuffer = function (map) {
        var _this = this;
        map.forEach(function (val, key) {
            _this.bufferedCalendar.set(key, val);
        });
    };
    Object.defineProperty(Timer.prototype, "getBufferedCalendar", {
        get: function () {
            return this.bufferedCalendar;
        },
        enumerable: true,
        configurable: true
    });
    Timer.prototype.getDayInfo = function (date) {
        if (!(date instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        var formatedDate = timer_utils_1.TimerUtils.getFormatedDate(date);
        var bufferedDate = this.getBufferedCalendar.get(formatedDate);
        if (!bufferedDate) {
            var daysBetween = timer_utils_1.TimerUtils.getDaysBetween(date, new Date());
            if (daysBetween > 0) {
                this.timerParams.maxBufferedDays = daysBetween + 5;
            }
            else {
                this.timerParams.minBufferedDays = daysBetween - 5;
            }
            this.constructWorkingDays(this.timerParams, true);
            bufferedDate = this.getBufferedCalendar.get(formatedDate);
        }
        return bufferedDate;
    };
    Timer.prototype.getDayInfoAsync = function (date) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.getDayInfo(date);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.workingTimeBetween = function (from, to, unit) {
        var _this = this;
        if (!(from instanceof Date || to instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        if (from.getTime() > to.getTime()) {
            throw new timer_error_1.default('From date should be less than to date !');
        }
        var count = 0;
        if (unit.toUpperCase() === 'DAYS') {
            var bufferedDate = this.getDayInfo(from);
            var nextDate = new Date(from.setHours(24, 0, 0, 0));
            while ((!(bufferedDate.isVacation || bufferedDate.isWeekend) &&
                !bufferedDate.isExceptional) &&
                nextDate < to) {
                count++;
                nextDate = new Date(from.setHours(24, 0, 0, 0));
                bufferedDate = this.getDayInfo(nextDate);
            }
            return count;
        }
        var nextWindow = this.getNextWorkingTime(from);
        if (nextWindow.getTime() > to.getTime()) {
            return 0;
        }
        var _loop_1 = function () {
            var day = timer_utils_1.TimerUtils.getFormatedDate(nextWindow);
            var bufferedDate = this_1.getDayInfo(nextWindow);
            bufferedDate.workingHours.forEach(function (window) {
                var startTime = new Date(day + " " + window.from).getTime();
                var endTime = new Date(day + " " + window.to).getTime();
                var nextWindowMs = nextWindow.getTime();
                if (nextWindowMs >= startTime &&
                    nextWindowMs <= endTime &&
                    nextWindowMs < to.getTime()) {
                    if (endTime < to.getTime()) {
                        count += (endTime - nextWindowMs);
                    }
                    else {
                        count += (to.getTime() - nextWindowMs);
                    }
                    nextWindow = _this.getNextWorkingTime(new Date(endTime));
                }
            });
        };
        var this_1 = this;
        while (nextWindow.getTime() <= to.getTime()) {
            _loop_1();
        }
        return (unit.toUpperCase() === 'MINUTES') ?
            Number((count / timer_unit_1.TimerUnit.MINUTES).toFixed(1)) :
            Number((count / timer_unit_1.TimerUnit.HOURS).toFixed(1));
    };
    Timer.prototype.workingTimeBetweenAsync = function (from, to, unit) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.workingTimeBetween(from, to, unit);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.isWorkingTime = function (date) {
        if (!(date instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        var bufferedDate = this.getDayInfo(date);
        if (bufferedDate.isVacation || (bufferedDate.isWeekend && !bufferedDate.isExceptional)) {
            return false;
        }
        var requestedTime = date.getTime();
        var day = timer_utils_1.TimerUtils.getFormatedDate(date);
        var inWindow = false;
        bufferedDate.workingHours.forEach(function (window) {
            var startTime = new Date(day + " " + window.from).getTime();
            var endTime = new Date(day + " " + window.to).getTime();
            if (requestedTime >= startTime &&
                requestedTime < endTime) {
                inWindow = true;
            }
        });
        return inWindow;
    };
    Timer.prototype.isWorkingTimeAsync = function (date) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.isWorkingTime(date);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.getNextWorkingTime = function (date) {
        if (!(date instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        var bufferedDate = this.getDayInfo(date);
        var nextWindow = null;
        if (bufferedDate.isVacation || (bufferedDate.isWeekend && !bufferedDate.isExceptional)) {
            var nextDay = new Date(date.getTime() + timer_unit_1.TimerUnit.DAYS).setHours(0, 0, 0, 0);
            return this.getNextWorkingTime(new Date(nextDay));
        }
        if (this.isWorkingTime(date)) {
            return date;
        }
        var requestedTime = date.getTime();
        var day = timer_utils_1.TimerUtils.getFormatedDate(date);
        bufferedDate.workingHours.forEach(function (window, index) {
            var startTime = new Date(day + " " + window.from).getTime();
            var endTime = new Date(day + " " + window.to).getTime();
            if (!nextWindow || nextWindow.getTime() > startTime) {
                if (requestedTime >= startTime &&
                    requestedTime < endTime) {
                    nextWindow = date;
                }
                else if (startTime > requestedTime) {
                    nextWindow = new Date(startTime);
                }
            }
        });
        if (!nextWindow) {
            var nextDay = new Date(date.getTime() + timer_unit_1.TimerUnit.DAYS).setHours(0, 0, 0, 0);
            return this.getNextWorkingTime(new Date(nextDay));
        }
        return nextWindow;
    };
    Timer.prototype.getNextWorkingTimeAsync = function (date) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.getNextWorkingTime(date);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.getPreviousWorkingTime = function (date) {
        if (!(date instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        var bufferedDate = this.getDayInfo(date);
        var previousWindow = null;
        if (bufferedDate.isVacation || (bufferedDate.isWeekend && !bufferedDate.isExceptional)) {
            var previousDay = new Date(date.setDate(date.getDate() - 1)).setHours(23, 59, 59, 0);
            return this.getPreviousWorkingTime(new Date(previousDay));
        }
        if (this.isWorkingTime(date)) {
            return date;
        }
        var requestedTime = date.getTime();
        var day = timer_utils_1.TimerUtils.getFormatedDate(date);
        for (var i = bufferedDate.workingHours.length - 1; i >= 0; i--) {
            var window = bufferedDate.workingHours[i];
            var startTime = new Date(day + " " + window.from).getTime();
            var endTime = new Date(day + " " + window.to).getTime();
            if (requestedTime >= startTime &&
                requestedTime < endTime) {
                previousWindow = date;
                break;
            }
            else if (endTime < requestedTime) {
                previousWindow = new Date(endTime);
                break;
            }
        }
        if (!previousWindow) {
            var previousDay = new Date(date.setDate(date.getDate() - 1)).setHours(23, 59, 59, 0);
            return this.getPreviousWorkingTime(new Date(previousDay));
        }
        return previousWindow;
    };
    Timer.prototype.getPreviousWorkingTimeAsync = function (date) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.getPreviousWorkingTime(date);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.getNextWorkingDay = function (date) {
        if (!(date instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        var tomorrow = new Date(date.setHours(24, 0, 0, 0));
        return this.getNextWorkingTime(tomorrow);
    };
    Timer.prototype.getNextWorkingDayAsync = function (date) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.getNextWorkingDay(date);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.getPreviousWorkingDay = function (date) {
        if (!(date instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        var prevDate = new Date(date.setHours(0, 0, 0, 0) - timer_unit_1.TimerUnit.DAYS);
        var dayInfo = this.getDayInfo(prevDate);
        if ((dayInfo.isVacation || dayInfo.isWeekend) && !dayInfo.isExceptional) {
            return this.getPreviousWorkingDay(prevDate);
        }
        else {
            return this.getNextWorkingTime(prevDate);
        }
    };
    Timer.prototype.getPreviousWorkingDayAsync = function (date) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.getPreviousWorkingDay(date);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.add = function (date, duration, unit) {
        var _this = this;
        if (!(date instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        var nextWindow = this.getNextWorkingTime(date);
        if (unit.toUpperCase() === 'DAYS') {
            for (var index = 1; index <= duration; index++) {
                var nextDate = new Date(nextWindow.setHours(0, 0, 0, 0) + timer_unit_1.TimerUnit.DAYS);
                nextWindow = this.getNextWorkingTime(nextDate);
            }
            return nextWindow;
        }
        var timerMs = (unit.toUpperCase() === 'MINUTES') ? duration * timer_unit_1.TimerUnit.MINUTES : duration * timer_unit_1.TimerUnit.HOURS;
        var _loop_2 = function () {
            var day = timer_utils_1.TimerUtils.getFormatedDate(nextWindow);
            var bufferedDate = this_2.getDayInfo(nextWindow);
            var nextWindowMs = nextWindow.getTime();
            bufferedDate.workingHours.forEach(function (window) {
                var startTime = new Date(day + " " + window.from).getTime();
                var endTime = new Date(day + " " + window.to).getTime();
                if (nextWindowMs == startTime) {
                    if (endTime - startTime > timerMs) {
                        nextWindow = new Date(startTime + timerMs);
                        timerMs = 0;
                    }
                    else {
                        nextWindow = _this.getNextWorkingTime(new Date(endTime));
                        timerMs = timerMs - (endTime - startTime);
                    }
                }
                else if (nextWindowMs > startTime && nextWindowMs < endTime) {
                    if (endTime - nextWindowMs > timerMs) {
                        nextWindow = new Date(nextWindowMs + timerMs);
                        timerMs = 0;
                    }
                    else {
                        nextWindow = _this.getNextWorkingTime(new Date(endTime));
                        timerMs = timerMs - (endTime - nextWindowMs);
                    }
                }
            });
        };
        var this_2 = this;
        while (timerMs !== 0) {
            _loop_2();
        }
        return nextWindow;
    };
    Timer.prototype.addAsync = function (date, duration, unit) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            var data;
                            try {
                                data = _this.add(date, duration, unit);
                                resolve(data);
                            }
                            catch (error) {
                                reject(error);
                            }
                        }, 0);
                    })];
            });
        });
    };
    Timer.prototype.setWorkingTimeout = function (baseDate, duration, unit, cb, desc) {
        if (!(baseDate instanceof Date)) {
            throw new timer_error_1.default('Invalid Date !');
        }
        if (this.bufferedCalendar.size === 0) {
            throw new timer_error_1.default('Please set configuration !');
        }
        var fireDate = this.add(baseDate, duration, unit);
        if (fireDate.getTime() < Date.now()) {
            fireDate = this.getNextWorkingTime(new Date());
        }
        return new working_timeout_1.default(baseDate, fireDate, duration, unit, cb, desc);
    };
    Timer.prototype.setWorkingTimeoutAsync = function (baseDate, duration, unit, cb, desc) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            setTimeout(function () {
                var data;
                try {
                    data = _this.setWorkingTimeout(baseDate, duration, unit, cb, desc);
                    resolve(data);
                }
                catch (error) {
                    reject(error);
                }
            }, 0);
        });
    };
    return Timer;
}());
exports.Timer = Timer;
